import logging
import traceback

from sqlite_influx import Config
from kacors485 import kacors485

logger = logging.getLogger(__name__)

def read_inverter(kaco, inverter_id):
    #read inverter with address i
    inv_data = kaco.readInverterAndParse(int(inverter_id))

    logger.info("read inverter {}: {}".format(inverter_id, inv_data))

    return inv_data

def read_all_inverter(wdog_manager):
    logger.info("read all inverters")

    measurements = {}
    try:
        kaco = kacors485.KacoRS485(Config.config['kaco_port'])

        # if we make it till here without Exception, RS485 port was found
        wdog_manager.update_result('port_found', True)

        inverters = Config.config['kaco_inverter_ids']
        for i in inverters:
            # read data
            data = read_inverter(kaco, i)

            # convert to measurement format
            measurement = 'inverter'+str(i)
            fields = {k:data[k]['value'] for k in data}

            # add to all measurements
            measurements[measurement] = fields

        kaco.close()

        # set communication info
        measurements['communication'] = {
            'last_request_status': 'ok'
        }
    except Exception:
        trace = traceback.format_exc()
        logger.exception("error while requesting inverters")

        # set communication info
        measurements['communication'] = {
            'last_request_status': 'error',
            'error_msg': trace
        }

    return measurements



import logging
import traceback
import sqlite3
from sqlalchemy import desc

from sqlite_influx import Config, sqlite

logger = logging.getLogger(__name__)

def get_rows():
    def dict_factory(cursor, row):
        d = {}
        for idx, col in enumerate(cursor.description):
            d[col[0]] = row[idx]
        return d

    conn = sqlite3.connect(Config.config['sbfspot_sqlite_file'])
    conn.row_factory = dict_factory
    c = conn.cursor()

    # TODO number of records hard coded, therfore we can only retrieve
    # these many different inverter data
    c.execute('SELECT * FROM SpotData Order by TimeStamp desc limit 5')
    data = c.fetchall()
    
    measurements = {}
    for d in data:
        # add inverter_ to metric name of inverter
        key = 'inverter_{}'.format(d['Serial'])
        if not key in measurements:
            measurements[key] = d

    return measurements

def read_all_inverter(wdog_manager):
    logger.info("read all inverters")

    measurements = {}
    try:
        data = get_rows()
        measurements.update(data)

        # if we make it till here without Exception, sbfspot stuff was found
        wdog_manager.update_result('port_found', True)

        # set communication info
        measurements['communication'] = {
            'last_request_status': 'ok'
        }
    except Exception:
        trace = traceback.format_exc()
        logger.exception("error while requesting inverters")

        # set communication info
        measurements['communication'] = {
            'last_request_status': 'error',
            'error_msg': trace
        }

    return measurements



import unittest as unittest
import datetime
import os
import time
try:
    from unittest import mock
except ImportError:
    import mock

from pv_logger import watchdog

class WatchdogTest(unittest.TestCase):
    def test_internet(self):
        internet = watchdog.test_internet_connection()

        self.assertEqual(internet, True)

    def test_uptime(self):
        uptime = watchdog.get_uptime()

        self.assertTrue(uptime > datetime.timedelta(seconds=5))

    def test_reboot(self):
        watchdog.reboot()

        print("cancel reboot again")
        os.system("shutdown -c")

    def test_freemem(self):
        mem = watchdog.get_freemem()

        self.assertTrue(isinstance(mem, int))

class WatchdogManagerTest(unittest.TestCase):
    callback_1_count = 0
    fail = True

    @mock.patch('pv_logger.watchdog.reboot')
    def test_ok_fail(self, reboot_mock):
        for self.fail in [False, True]:
            print("shoudl fail?", self.fail)
            self.callback_1_count = 0
            wdog_manager = watchdog.Manager()
            # set min uptime of watchdog to 0, so reboot is always called
            wdog_manager.min_uptime_for_reboot = datetime.timedelta(
                seconds=0)

            wdog_manager.register_check('callback_1',
                datetime.timedelta(seconds=0.2), 3, self.callback_1)

            # testing of 3 fails takes 0.2*2 = 0.4 seconds
            # run at 0.s0, 0.1s, 0.2s, 0.3s, 0.4s
            for i in range(1,6):
                wdog_manager.run()
                # check callback is only called when interval is over
                print("waited since start:", (i-1)*0.1)
                if i in [1, 2]:
                    self.assertEqual(self.callback_1_count, 1)
                if i in [3, 4]:
                    self.assertEqual(self.callback_1_count, 2)
                time.sleep(0.2)

            self.assertEqual(reboot_mock.call_count, int(self.fail))

    @mock.patch('pv_logger.watchdog.get_uptime')
    @mock.patch('pv_logger.watchdog.reboot')
    def test_no_reboot_not_enough_uptime(self, reboot_mock, uptime_mock):
        uptime_mock.return_value = datetime.timedelta(seconds=10)

        self.fail = True
        wdog_manager = watchdog.Manager()
        wdog_manager.register_check('callback_1',
            datetime.timedelta(seconds=0.1), 2, self.callback_1)

        for i in [1,2]:
            wdog_manager.run()
            time.sleep(0.1)

        self.assertEqual(reboot_mock.call_count, 0)

    @mock.patch('pv_logger.watchdog.reboot')
    def test_check_result_not_set(self, reboot_mock):
        wdog_manager = watchdog.Manager()
        # set min uptime of watchdog to 0, so reboot is always called
        wdog_manager.min_uptime_for_reboot = datetime.timedelta(
            seconds=0)

        wdog_manager.register_check('check_manual',
            datetime.timedelta(seconds=0.2), 2)

        # set result once
        wdog_manager.update_result('check_manual', True)

        time.sleep(0.2)
        # set again
        wdog_manager.update_result('check_manual', True)
        wdog_manager.run()
        self.assertEqual(reboot_mock.call_count, 0)

        time.sleep(0.3)
        # now not updated too long
        wdog_manager.run()
        self.assertEqual(reboot_mock.call_count, 1)

    @mock.patch('pv_logger.watchdog.reboot')
    def test_check_result_history(self, reboot_mock):
        """check that test result history is stored correctly
        and that always the latest x results are kept, in order,
        and no others..."""
        wdog_manager = watchdog.Manager()
        # set min uptime of watchdog to 0, so reboot is always called
        wdog_manager.min_uptime_for_reboot = datetime.timedelta(
            seconds=0)

        wdog_manager.register_check('check_manual',
            datetime.timedelta(seconds=0.4), 3)

        # set result once
        wdog_manager.update_result('check_manual', True)
        time.sleep(0.1)
        wdog_manager.update_result('check_manual', False)
        time.sleep(0.1)
        wdog_manager.update_result('check_manual', True)
        time.sleep(0.1)

        wdog_manager.run()
        wdog_manager.update_result('check_manual', False)
        wdog_manager.run()

        # check that last update_result is False
        checks = wdog_manager.checks
        last_check_result = checks['check_manual']['results'][-1][-1]
        self.assertEqual(last_check_result, False)

    def callback_1(self):
        # if we want to fail, return False (check function failed)
        self.callback_1_count += 1
        return not self.fail



"""watch some system metrics"""

import datetime
import socket
from urllib2 import urlopen
import logging
import os

from sqlite_influx import sqlite, Config

logger = logging.getLogger(__name__)

def test_internet_connection():
    """test internet connection

    Returns:
        working (bool): If internet is working, return True
    """

    socket.setdefaulttimeout( 23 )
    try:
        response = urlopen('http://google.com/')
        return True
    except Exception as e:
        logger.exception("seems like no internet connection")
        return False

def get_freemem():
    """get free ram in kbytes of computer"""
    mem = os.popen("awk '/MemFree/{print $2}' /proc/meminfo").read()
    return int(mem)

def get_uptime():
    """get uptime of computer

    Returns:
        uptime (:obj:`datetime.timedelta`)
    """

    with open('/proc/uptime', 'r') as f:
        uptime_seconds = float(f.readline().split()[0])
        uptime = datetime.timedelta(seconds = uptime_seconds)

        return uptime


def reboot():
    """try to reboot the system in 3 minutes

    Returns:
        success (bool): If reboot command was successfull
    """
    # shutdown now, rpi has no realtime clock
    # therefore future shutdowns might become a problem
    res = os.system("sudo -n shutdown -r now")
    if res == 0:
        return True
    else:
        return False

class Manager(object):
    """register checks which will be overseen
    if check fails, then system will reboot, if other conditios are met

    Example:
        wdog_manager = Manager()
        wdog_manager.register_check('internet_connection',
            datetime.timedelta(seconds=30), 3, test_internet_connection)

        while True:
            wdog_manager.run()
            time.sleep(30)
    """

    def __init__(self):
        # use uptime, because it also works on read only systems
        self.start_time = get_uptime()
        logger.info('uptime at app start is: {}'.format(
                self.start_time
            ))

        self.checks = {}

        if 'min_uptime_seconds_before_reboot' in Config.config:
            min_seconds = Config.config['min_uptime_seconds_before_reboot']
        else:
            logger.info('use default min uptime seconds before reboot of 3600'
                ' because not found in config')
            min_seconds = 3600

        min_uptime_for_reboot = datetime.timedelta(
            seconds=min_seconds)
        assert isinstance(min_uptime_for_reboot, datetime.timedelta)
        self.min_uptime_for_reboot = min_uptime_for_reboot

    def register_check(self, name, interval, min_fails_in_row, callback=None):
        item = {
            'interval': interval,
            'min_fails_in_row': min_fails_in_row,
            'results': [],
            'callback': callback
        }
        self.checks[name] = item

    def update_checks(self):
        for name in self.checks:
            check = self.checks[name]

            # update value
            if check['callback']:
                # but only if interval is over
                now = get_uptime()

                # check if not first run
                if len(check['results']) < 1:
                    interval_over = True
                else:
                    last_time = check['results'][-1][0]
                    interval_over = (last_time + check['interval'] < now)

                if interval_over:
                    # interval since last check is over
                    # do update result
                    value = check['callback']()
                    self.update_result(name, value)

            # keep only latest x results
            x = check['min_fails_in_row']
            self.checks[name]['results'] = self.checks[name]['results'][-x:]

    def update_result(self, name, value):
        """update result of a check

        must be done manually, if no callback was provided
        """
        check = (get_uptime(), bool(value))
        self.checks[name]['results'].append(check)
        logger.info("update watchdog result of %s with %s", name, check)

    def log_reboot(self, failed):
        """log reboot reason"""

        message = 'Reboot system because of failed checks:\n'
        message += '\n'.join([item[1] for item in failed])

        logger.error(message)
        measurements = {
            'watchdog_message': {
                'message': message
            }
        }
        sqlite.store_dicts(measurements)

    def run(self):
        logger.debug("manager:run()")
        self.update_checks()

        failed = self.find_failed()
        if len(failed) > 0:
            # check other conditions
            uptime = get_uptime()
            logger.info("uptime is {}, {} needed before reboot".format(
                uptime, self.min_uptime_for_reboot))
            if uptime > self.min_uptime_for_reboot:
                # now we can reboot
                self.log_reboot(failed)
                reboot()

    def find_failed(self):
        """find failed checks

        Returns:
            failed_checks (list of tuples): list of tuples (name, message)
        """
        failed_checks = []
        for name in self.checks:
            results = self.checks[name]['results']
            logger.debug("results of check {} are {}".format(name, results))

            # we only keep no of min_fails_in_row results
            # therefore we need only count how many of them are failed
            failed_count = sum([1 for item in results if item[1] is False])

            if failed_count >= self.checks[name]['min_fails_in_row']:
                # test failed too often
                message = ("check {} failed due to {} fails in a row".format(
                    name, failed_count))
                failed_checks.append((name, message))

                # ignore other checks
                continue

            # check can also fail, if not updated too long
            if len(results) > 0:
                last_time = results[-1][0]
                used_start_time = ''
            else:
                last_time = self.start_time
                used_start_time = ' never, checked against start time instead '

            now = get_uptime()
            interval_over = (last_time + self.checks[name]['interval'] < now)
            if interval_over:
                # test not updated for too long
                message = ("check {} failed due to missing "
                            "update in interval. Should update every "
                            "{}, but last was {}{}".format(
                            name, self.checks[name]['interval'],
                            used_start_time, last_time))
                failed_checks.append((name, message))

        return failed_checks



import unittest as unittest
import serial
import os
import time
try:
    from unittest import mock
except ImportError:
    import mock

from pv_logger import logger as pvlogger

pvlogger.Config.config['inverter_type'] = ['kaco']

class HighlevelTest(unittest.TestCase):
    def get_rows(self):
        rows = pvlogger.sqlite.session.query(pvlogger.sqlite.History).all()
        rows_converted = [pvlogger.sqlite.history_to_dict(row) for row in rows]
        for item in rows_converted:
            print("row in db", item)

        return rows_converted

    def remove_all_rows(self):
        pvlogger.sqlite.session.query(pvlogger.sqlite.History).delete()

    def test_loop(self):
        pvlogger.main_loop()

        rows = self.get_rows()
        self.remove_all_rows()

        self.assertEqual(rows[0]['fields']['last_request_status'], 'error')
        self.assertEqual(len(rows), 2)

        module_loaded = ('kaco.py' in repr(rows[0]['fields']))
        self.assertEqual(module_loaded, True)


    @mock.patch('serial.Serial', spec=serial.Serial)
    def test_answers(self, serial_mock):
        dir_path = os.path.dirname(os.path.realpath(__file__))
        pvlogger.Config.config['kaco_port'] = os.path.join(dir_path, 'test_ttyUSB*')

        # reset mock state
        self.serial_mock_in_waiting_index = 0
        self.serial_mock_return_index = 0

        instance = serial_mock.return_value
        instance.inWaiting.side_effect = self.serial_mock_in_waiting
        instance.readline.side_effect = self.serial_mock_return

        pvlogger.main_loop()

        rows = self.get_rows()
        self.remove_all_rows()

        # assume that if 3 answers and one status is written
        # everything was working
        self.assertEqual(len(rows), 5)

    @mock.patch('serial.Serial', spec=serial.Serial)
    def test_answers_sometimes(self, serial_mock):
        """check if inverter answers sometimes and watchdog is set correctly"""
        dir_path = os.path.dirname(os.path.realpath(__file__))
        pvlogger.Config.config['kaco_port'] = os.path.join(dir_path, 'test_ttyUSB*')
        pvlogger.Config.config['checks']['port_found']['interval_seconds'] = 0.1

        # reset mock state
        self.serial_mock_in_waiting_index = 0
        self.serial_mock_return_index = 0

        self.remove_all_rows()


        # run twice
        for i in range(1,3):
            pvlogger.main_loop()
            rows = self.get_rows()
            self.remove_all_rows()

            # one for no inverter answer, one for no communication to influx
            self.assertEqual(len(rows), 2)

        # now fail
        # TODO: fail not working
        # TODO: reproduce issue of pv_strass!!!
        #
        time.sleep(0.4)
        pvlogger.Config.config['kaco_port'] = '/tmp/not-here-asdfasdf'

        pvlogger.main_loop()
        rows = self.get_rows()
        self.remove_all_rows()

        # one for no inverter answer, one for no communication to influx
        self.assertEqual(len(rows), 2)



    def serial_mock_in_waiting(self):
        """return 1 or 0 based on number of calls"""
        # each inverter will be asked 2 questions
        # all together are 2*num of inverters
        should_return = [1, 0]*6

        index = self.serial_mock_in_waiting_index
        self.serial_mock_in_waiting_index += 1

        if index > len(should_return) - 1:
            return 0
        else:
            return should_return[index]

    def serial_mock_return(self):
        """return answer based on number of calls"""
        should_return = [
            '*010 4 585.9 10.17 5958 229.5 24.90 5720 36 17614 d 9600I',
            '2286 4184 42 581 8:46 11:04 11:04',
            '*020 4 585.9 10.17 5958 229.5 24.90 5720 36 17614 d 9600I',
            '2286 4184 42 581 8:46 11:04 11:04',
            '*030 4 585.9 10.17 5958 229.5 24.90 5720 36 17614 d 9600I',
            '2286 4184 42 581 8:46 11:04 11:04',
        ]

        index = self.serial_mock_return_index
        self.serial_mock_return_index += 1

        return should_return[index]



"""read data from kacors485, cache them locally in sqlite
and finally sync them to remote InfluxDB server (assume local ssh tunnel)"""

import yaml
import logging
import time
import os
import datetime

from sqlite_influx import Config

# setup logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

# setup config first, before we import the other stuff
config_file = 'config.yaml'
with open(config_file, 'r') as stream:
    logger.info("read config file {}".format(os.path.abspath(config_file)))
    config_yaml = yaml.load(stream)
Config.config.update(config_yaml)

if 'log_level' in Config.config:
    defined_level = Config.config['log_level']
    pre_defined_level = ['CRITICAL', 'ERROR', 'WARNING', 'INFO', 'DEBUG']
    if defined_level in pre_defined_level:
        level = logging.getLevelName(defined_level)
        logging.basicConfig(level = level)
        logging.getLogger().setLevel(level = level)
        logging.info("set new log level from config file to: {} ({})".
            format(level, defined_level))
    else:
        logging.warning("log level in config file not valid: {}".format(level))

# now import helpers, which initialize database
# based on config data
from sqlite_influx import sqlite
from pv_logger import watchdog
from pv_logger import kaco, sma_rs485, sbfspot

wdog_manager = watchdog.Manager()
# setup checks at import, because also in functions used (not only in main)
check_conf = Config.config['checks']['internet_connection']
wdog_manager.register_check('internet_connection',
    datetime.timedelta(seconds=check_conf['interval_seconds']),
    min_fails_in_row=check_conf['min_fails_in_row'],
    callback=watchdog.test_internet_connection)

check_conf = Config.config['checks']['port_found']
wdog_manager.register_check('port_found',
    datetime.timedelta(seconds=check_conf['interval_seconds']),
    min_fails_in_row=check_conf['min_fails_in_row'])


def measurements_remove_empty(measurements):
    return {k: measurements[k] for k in measurements if measurements[k]}

def read_data():
    inverter_types = Config.config['inverter_type']

    # convert to list, if not yet
    if not isinstance(inverter_types, list):
        raise Exception('config.yaml "inverter_type" must be a list')

    measurements = {}
    for inverter_type in inverter_types:
        if inverter_type == 'kaco':
            data = kaco.read_all_inverter(wdog_manager)
        elif inverter_type == 'sma_rs485':
            data = sma_rs485.read_all_inverter(wdog_manager)
        elif inverter_type == 'sbfspot':
            data = sbfspot.read_all_inverter(wdog_manager)
        else:
            raise Exception('unknown "inverter_type" {} in configuration'.
                format(inverter_type))

        measurements.update(data)

    # clear measurements, where fields are empty
    measurements = measurements_remove_empty(measurements)

    return measurements

def main_loop():
    # get measurements
    measurements = read_data()

    # append more information
    uptime_now = watchdog.get_uptime()
    uptime_app = uptime_now - wdog_manager.start_time
    measurements['system'] = {
        'uptime_node': float(uptime_now.total_seconds()),
        'uptime_app': float(uptime_app.total_seconds()),
        'ram_free_kb': float(watchdog.get_freemem()),
    }

    # store measurements
    sqlite.store_dicts(measurements)

    sqlite.sync_to_influx()
    sqlite.archive(Config.config['archive_older_than_days'])

    # run watchdog manager
    wdog_manager.run()

def main():
    # run main loop
    while True:
        start = watchdog.get_uptime()

        main_loop()

        end = watchdog.get_uptime()
        logger.info("last main loop took %s seconds to run",
            end-start)

        time.sleep(Config.config['read_interval_seconds'])

if __name__ == '__main__':
    main()



import unittest as unittest
import os
try:
    from unittest import mock
except ImportError:
    import mock

from pv_logger import logger as pvlogger

pvlogger.Config.config['inverter_type'] = ['sbfspot']

dir_path = os.path.dirname(os.path.realpath(__file__))
pvlogger.Config.config['sbfspot_sqlite_file'] =  os.path.join(dir_path, 'test_sbfspot.db')

class HighlevelTest(unittest.TestCase):
    def get_rows(self):
        rows = pvlogger.sqlite.session.query(pvlogger.sqlite.History).all()
        rows_converted = [pvlogger.sqlite.history_to_dict(row) for row in rows]
        for item in rows_converted:
            print("row in db", item)

        return rows_converted

    def remove_all_rows(self):
        pvlogger.sqlite.session.query(pvlogger.sqlite.History).delete()

    def test_loop(self):
        self.remove_all_rows()
        pvlogger.main_loop()
        rows = self.get_rows()
        self.remove_all_rows()

        print("rows are", rows)
        self.assertEqual(len(rows), 3)



import unittest as unittest
try:
    from unittest import mock
except ImportError:
    import mock

from pv_logger import logger as pvlogger

pvlogger.Config.config['inverter_type'] = ['sma_rs485']

class HighlevelTest(unittest.TestCase):
    def get_rows(self):
        rows = pvlogger.sqlite.session.query(pvlogger.sqlite.History).all()
        rows_converted = [pvlogger.sqlite.history_to_dict(row) for row in rows]
        for item in rows_converted:
            print("row in db", item)

        return rows_converted

    def remove_all_rows(self):
        pvlogger.sqlite.session.query(pvlogger.sqlite.History).delete()

    def test_loop(self):
        pvlogger.main_loop()

        rows = self.get_rows()
        self.remove_all_rows()

        self.assertEqual(rows[0]['fields']['last_request_status'], 'error')
        self.assertEqual(len(rows), 2)

        sma_rs485_loaded = ('sma_rs485.py' in repr(rows[0]['fields']))
        self.assertEqual(sma_rs485_loaded, True)

    @mock.patch('pv_logger.sma_rs485.read_all_inverter')
    def test_write(self, read_mock):
        # mock answer, check if it can be written to influxdb
        mock_data = {
            'WR6K-006 SN:2000141813': {
                'T-Start-Fan': 70.0,
                'Upv-Start': 300.0,
            },
            'WR42MS04 SN:1100067554': {
                'Iac': 0,
                'Pac': 0,
                'Firmware-BFR': 0,
                'Netz-Ein': 0,
            }
        }
        read_mock.return_value = mock_data

        self.remove_all_rows()
        pvlogger.main_loop()
        rows = self.get_rows()

        print("rows are", rows)
        self.assertEqual(len(rows), 3)

        # check that it passed to influxdb
        influx = pvlogger.sqlite.influx.client.query("SELECT * FROM /WR42*/ ORDER BY time DESC LIMIT 1")
        influx_item = None
        for f in influx:
            influx_item = f[0]

        print("influx item", influx_item)
        print("WARNING: this test needs a influxdb backend to pass!!")
        self.assertEqual(influx_item['Firmware-BFR'], 0)



import logging
import traceback
import os

from sqlite_influx import Config
from pyyasdi.objects import Plant

logger = logging.getLogger(__name__)

def read_all_inverter(wdog_manager):
    plant = None

    logger.info("read all inverters")

    measurements = {}
    try:
        if not os.path.exists('yasdi.ini'):
            raise Exception('could not find yasdi.ini')

        if not isinstance(plant, Plant):
            logger.info("initialize plant yasdi RS485 instance")
            plant = Plant(debug=1,
                max_devices=Config.config['sma_rs485_max_devices'])
        else:
            logger.info("reset plant yasdi RS485 instance")
            plant.reset()

        # if we make it till here without Exception, RS485 port was found
        wdog_manager.update_result('port_found', True)

        data = plant.data_all(parameter_channel=False, skip_on_timeout=True)

        # stop plant
        plant.quit()

        # add inverter_ to inverter metric name
        renamed = {}
        for k in data:
            renamed['inverter_{}'.format(k)] = data[k]

        measurements.update(renamed)

        # set communication info
        measurements['communication'] = {
            'last_request_status': 'ok'
        }
    except Exception:
        trace = traceback.format_exc()
        logger.exception("error while requesting inverters")

        # set communication info
        measurements['communication'] = {
            'last_request_status': 'error',
            'error_msg': trace
        }

    return measurements



import unittest as unittest
import os
try:
    from unittest import mock
except ImportError:
    import mock

from pv_logger import logger as pvlogger

class GeneralTest(unittest.TestCase):
    def test_inverter_type_unknown(self):
        pvlogger.Config.config['inverter_type'] = ['unknown']

        try:
            pvlogger.main_loop()
        except Exception as e:
            if 'unknown' in str(e):
                pass
            else:
                raise
    def test_inverter_type_nolist(self):
        pvlogger.Config.config['inverter_type'] = 'nolist'

        try:
            pvlogger.main_loop()
        except Exception as e:
            if 'a list' in str(e):
                pass
            else:
                raise

    def test_remove_empty(self):
        first = {'node': 5960.91, 'app': 19.019097},

        data = {
                'uptime': first,
                'testempty': {}
            }

        res = pvlogger.measurements_remove_empty(data)
        self.assertEqual(res, {'uptime': first})

    def test_default_config_read(self):
        pvlogger.read_data()
